/*
 * Copyright 2013 Mark Slater
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */

package net.sourceforge.xazzle.xhtml;

import net.sourceforge.writexml.Attribute;
import net.sourceforge.writexml.Attributes;
import net.sourceforge.writexml.TagName;

import java.util.Collections;

import static java.util.Arrays.asList;
import static net.sourceforge.writexml.Attributes.attributes;
import static net.sourceforge.writexml.TagName.tagName;
import static net.sourceforge.xazzle.xhtml.CollectionUtils.newList;

public final class BlockElement<T extends MayContainFormTag>
        extends StandardXhtmlTag<BlockElement<T>, BlockTagContents<? extends T>>
        implements XhtmlTag, BodyElement<T, TagScope.Block> {

    BlockElement(final TagName tagName, final Attributes attributes, final Iterable<? extends BlockTagContents<? extends T>> children) {
        super(tagName, attributes, children);
    }

    static <T extends MayContainFormTag> BlockElement<T> divTag(final BodyElement<? extends T, ?>... bodyElements) {
        return divTag(asList(bodyElements));
    }

    static <T extends MayContainFormTag> BlockElement<T> divTag(final Iterable<BodyElement<? extends T, ?>> bodyElements) {
        return createBlockLevelTag(tagName("div"), bodyElements);
    }

    static BlockElement<DoesNotContainFormTag> paragraphTag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return paragraphTag(asList(children));
    }

    static BlockElement<DoesNotContainFormTag> paragraphTag(Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children1) {
        return createBlockLevelTag(tagName("p"), children1);
    }

    static <T extends MayContainFormTag> BlockElement<T> unorderedListTag(final ListItemTag<? extends T> listItemTag, final ListItemTag<? extends T>... listItemTags) {
        return unorderedListTag(listItemTag, asList(listItemTags));
    }

    static <T extends MayContainFormTag> BlockElement<T> unorderedListTag(final ListItemTag<? extends T> listItemTag, final Iterable<ListItemTag<? extends T>> listItemTags) {
        return createBlockLevelTagWithAtLeastOneChild(tagName("ul"), listItemTag, listItemTags);
    }

    static <T extends MayContainFormTag> BlockElement<T> orderedListTag(final ListItemTag<? extends T> listItemTag, final ListItemTag<? extends T>... listItemTags) {
        return orderedListTag(listItemTag, asList(listItemTags));
    }

    static <T extends MayContainFormTag> BlockElement<T> orderedListTag(final ListItemTag<? extends T> listItemTag, final Iterable<ListItemTag<? extends T>> listItemTags) {
        return createBlockLevelTagWithAtLeastOneChild(tagName("ol"), listItemTag, listItemTags);
    }

    static <T extends MayContainFormTag> BlockElement<T> definitionListTag(final DefinitionListContentTag<? extends T> definitionListContentTag, final DefinitionListContentTag<? extends T>... definitionListContentTags) {
        return definitionListTag(definitionListContentTag, asList(definitionListContentTags));
    }

    static <T extends MayContainFormTag> BlockElement<T> definitionListTag(final DefinitionListContentTag<? extends T> listItemTag, final Iterable<DefinitionListContentTag<? extends T>> listItemTags) {
        return createBlockLevelTagWithAtLeastOneChild(tagName("dl"), listItemTag, listItemTags);
    }

    static BlockElement<DoesNotContainFormTag> addressTag(final BodyElement<? extends DoesNotContainFormTag, ? extends TagScope.Inline>... inlineElements) {
        return addressTag(asList(inlineElements));
    }

    static BlockElement<DoesNotContainFormTag> addressTag(final Iterable<BodyElement<? extends DoesNotContainFormTag, ? extends TagScope.Inline>> inlineElements) {
        return createBlockLevelTag(tagName("address"), inlineElements);
    }

    static <T extends MayContainFormTag> BlockElement<T> blockQuoteTag(final BodyElement<? extends T, ? extends TagScope.Block>... children) {
        return blockQuoteTag(asList(children));
    }

    static <T extends MayContainFormTag> BlockElement<T> blockQuoteTag(final Iterable<? extends BodyElement<? extends T, ? extends TagScope.Block>> children) {
        return createBlockLevelTag(tagName("blockquote"), children);
    }

    static BlockElement<DoesNotContainFormTag> horizontalRuleTag() {
        return createBlockLevelTag(tagName("hr"), Collections.<BodyElement<DoesNotContainFormTag, TagScope>>emptyList());
    }

    private static <T extends MayContainFormTag> BlockElement<T> createBlockLevelTag(final TagName tagName, Iterable<? extends BodyElement<? extends T, ? extends TagScope>> children) {
        return new BlockElement<>(tagName, attributes(), children);
    }

    private static <T extends MayContainFormTag> BlockElement<T> createBlockLevelTagWithAtLeastOneChild(final TagName tagName, final BlockTagContents<? extends T> tag, final Iterable<? extends BlockTagContents<? extends T>> tags) {
        return new BlockElement<>(tagName, attributes(), IterableUtils.cons(tag, newList(tags)));
    }

    static BlockElement<DoesNotContainFormTag> h1Tag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return h1Tag(asList(children));
    }

    static BlockElement<DoesNotContainFormTag> h2Tag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return h2Tag(asList(children));
    }

    static BlockElement<DoesNotContainFormTag> h3Tag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return h3Tag(asList(children));
    }

    static BlockElement<DoesNotContainFormTag> h4Tag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return h4Tag(asList(children));
    }

    static BlockElement<DoesNotContainFormTag> h5Tag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return h5Tag(asList(children));
    }

    static BlockElement<DoesNotContainFormTag> h6Tag(final BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>... children) {
        return h6Tag(asList(children));
    }

    static BlockElement<DoesNotContainFormTag> h1Tag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return createHeadingTag(tagName("h1"), children);
    }

    static BlockElement<DoesNotContainFormTag> h2Tag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return createHeadingTag(tagName("h2"), children);
    }

    static BlockElement<DoesNotContainFormTag> h3Tag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return createHeadingTag(tagName("h3"), children);
    }

    static BlockElement<DoesNotContainFormTag> h4Tag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return createHeadingTag(tagName("h4"), children);
    }

    static BlockElement<DoesNotContainFormTag> h5Tag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return createHeadingTag(tagName("h5"), children);
    }

    static BlockElement<DoesNotContainFormTag> h6Tag(final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return createHeadingTag(tagName("h6"), children);
    }

    private static BlockElement<DoesNotContainFormTag> createHeadingTag(final TagName tagName, final Iterable<? extends BodyElement<DoesNotContainFormTag, ? extends TagScope.Inline>> children) {
        return createBlockLevelTag(tagName, children);
    }

    static <T extends MayContainFormTag> BlockElement<T> noScriptTag(final BodyElement<? extends T, ? extends TagScope.Block>... children) {
        return noScriptTag(asList(children));
    }

    static <T extends MayContainFormTag> BlockElement<T> noScriptTag(final Iterable<? extends BodyElement<? extends T, ? extends TagScope.Block>> children) {
        return createBlockLevelTag(tagName("noscript"), children);
    }

    @Override
    BlockElement<T> makeANewOne(final Attribute attribute) {
        return new BlockElement<>(tagName, attributes.append(attribute), children);
    }
}
